from error import FileNotWritableError, FolderAlreadyExistsError, FolderNotFoundError, CreateFolderError
import os
from shutil import copytree
import sys
import re
from pkg_resources import resource_filename


def we_are_frozen():
    # All of the modules are built-in to the interpreter, e.g., by py2exe
    return hasattr(sys, "frozen")


def get_path():
    encoding = sys.getfilesystemencoding()
    if we_are_frozen():
        return os.path.dirname(unicode(sys.executable, encoding))
    return os.path.dirname(unicode(__file__, encoding))


class New:
    def __init__(self, projectName, plugin=None, type='default'):
        self._projectName = projectName
        self._root = get_path()
        self._cwd = os.getcwd()
        self._plugin = plugin
        self._type = type

        if plugin:
            try:
                self._skeleton_path = plugin.skeleton_path()
            except NotImplementedError:
                try:
                    self._skeleton_path = resource_filename(__name__, os.path.join('skeleton', 'default'))
                except NotImplementedError:
                    self._skeleton_path = os.path.join(sys.prefix, 'skeleton', 'default')
        else:
            try:
                self._skeleton_path = resource_filename(__name__, os.path.join('skeleton', 'default'))
            except NotImplementedError:
                self._skeleton_path = os.path.join(sys.prefix, 'skeleton', 'default')

        self._projectPath = os.path.join(self._cwd, self._projectName)
        self._deployment_path = os.path.join(os.path.expanduser('~'))
        self._zip_path = os.path.join(os.path.expanduser('~'))

        if sys.platform.startswith('win32'):
            self._deployment_path = self._deployment_path.replace('\\', '\\\\')
            self._zip_path = self._zip_path.replace('\\', '\\\\')

        try:
            self._copy_structure()
        except:
            raise

        try:
            self._replace_strings()
        except:
            raise

    def _copy_structure(self):
        if os.path.exists(os.path.join(self._cwd, self._projectName)):
            raise FolderAlreadyExistsError('There is already a folder with the projectname present!')

        if not os.path.exists(self._skeleton_path):
            raise FolderNotFoundError('Could not find the skeleton: ', self._skeleton_path)

        try:
            copytree(self._skeleton_path, os.path.join(self._cwd, self._projectName))
        except:
            raise CreateFolderError('Could not create the folders for the new project.')

    def _replace_strings(self):
        file_list = []
        for path, dirs, files in os.walk(self._projectPath):
            for f in files:
                file_list.append({
                    'file': f,
                    'path': path
                })

        for entry in file_list:
            if re.search('_X.', entry['file']):
                try:
                    self._replace(entry)
                except:
                    raise

    def _replace(self, fileObject):
        f = fileObject['file']
        p = fileObject['path']

        outfilename = f.replace('_X', '')
        with open(os.path.join(p, outfilename), 'w+') as out:
            infile = open(os.path.join(p, f))
            for line in infile:
                newline = line.replace('#DEPLOYMENTPATH', self._deployment_path)
                newline = newline.replace('#ZIPPATH', self._zip_path)
                newline = newline.replace('#PROJECTNAME', self._projectName)

                if self._plugin:
                    newline = self._plugin.new_replace_line(newline)

                out.write(newline)

            infile.close()

        try:
            os.remove(os.path.join(p, f))
        except:
            raise FileNotWritableError('Could not delete the initial replace file.')



import os
from error import FileNotFoundError, CreateFolderError, FileNotWritableError, RemoveFolderError, RemoveFileError, SassError
from shutil import copy2, copytree, rmtree
import re
import sass
import sys


class Build:
    def __init__(self, config):
        self._cwd = os.getcwd()
        self._config = config

    def build_project(self, restrict):
        self._project_build_style_path = os.path.join(self._config['build_path'], 'style')

        if not os.path.exists(self._config['build_path']):
            try:
                os.makedirs(self._config['build_path'])
            except:
                raise CreateFolderError('Could not create the project folder.')

        if restrict:
            for r in restrict:
                try:
                    if r == 'js':
                        self._build_javascript()
                    if r == 'css':
                        self._build_css()
                    if r == 'html':
                        self._build_html()
                    if r == 'img':
                        self._build_images()
                    if r == 'lib':
                        self._build_libraries()
                except:
                    raise
        else:
            try:
                self._build_javascript()
                self._build_css()
                self._build_html()
                self._build_images()
                self._build_libraries()
            except:
                raise

    def _build_javascript(self):
        source = os.path.join(self._cwd, 'src', 'application.js')
        dest = os.path.join(self._config['build_path'], 'application.js')

        if not os.path.exists(source):
            return

        try:
            self._js_string = self._concat_javascript()
        except:
            raise

        if os.path.exists(dest):
            try:
                os.remove(dest)
            except:
                raise RemoveFileError('Could not delete the existing javascript application file.')

        try:
            f = open(dest, 'w+')
        except:
            raise FileNotWritableError('Could not write the javascript file.')

        f.write(self._js_string)
        f.close()

    def _concat_javascript(self):
        f = None
        lines = []

        try:
            f = open(os.path.join(self._cwd, 'src', 'application.js'))
        except:
            raise FileNotFoundError('The specified file does not exist: ', os.path.join('src', 'application.js'))

        self._included_js_files = []
        try:
            lines = self._gather_javascript_lines(f)
        except FileNotFoundError:
            raise
        finally:
            f.close()

        return ''.join(lines)

    def _gather_javascript_lines(self, f):
        lines = []

        for line in f:
            match = re.match('\/\/= require ([a-zA-Z\/]+)', line)
            if match:
                sub_f = None

                sub_path = match.group(1)
                if sys.platform.startswith('win32'):
                    sub_path = sub_path.replace('/', '\\')
                sub_path = os.path.join(self._cwd, 'src', 'javascript', sub_path)
                sub_path = sub_path + '.js'

                if sub_path not in self._included_js_files:
                    self._included_js_files.append(sub_path)

                    try:
                        sub_f = open(sub_path)
                    except:
                        raise FileNotFoundError('The specified file does not exist: ', sub_path)

                    try:
                        lines = lines + self._gather_javascript_lines(sub_f)
                    except FileNotFoundError:
                        raise
                    finally:
                        sub_f.close()
            else:
                lines.append(line)

        return lines

    def _build_html(self):
        source = os.path.join(self._cwd, 'src', 'index.html')
        dest = os.path.join(self._config['build_path'], 'index.html')

        if not os.path.exists(source):
            return

        if os.path.exists(dest):
            try:
                os.remove(dest)
            except:
                raise RemoveFileError('Could not remove the existing html build file.')

        try:
            copy2(source, dest)
        except:
            raise FileNotWritableError('Could not write the html file.')

    def _build_css(self):
        source_scss = os.path.join(self._cwd, 'src', 'style', 'style.scss')
        source_css = os.path.join(self._cwd, 'src', 'style', 'style.css')
        dest = os.path.join(self._config['build_path'], 'style', 'style.css')

        if not os.path.exists(source_css) and not os.path.exists(source_scss):
            return

        if not os.path.exists(self._project_build_style_path):
            try:
                os.makedirs(self._project_build_style_path)
            except:
                raise CreateFolderError('Could not create the style folder.')

        if os.path.exists(source_css):
            try:
                os.remove(dest)
            except:
                raise RemoveFileError('Could not remove the existing css file.')

        if os.path.exists(source_scss):
            try:
                _css_string = sass.compile(filename=source_scss)
            except sass.CompileError as e:
                raise SassError('Could not compile your scss file:\n', e.message[:-1])
            except:
                raise FileNotFoundError('Could not find your scss style file.')

            try:
                f = open(dest, 'w+')
            except:
                raise FileNotWritableError('Could not write the new css file.')

            f.write(_css_string)
            f.close()
        elif os.path.exists(source_css):
            try:
                copy2(source_css, dest)
            except:
                raise FileNotWritableError('Could not write the new css file.')

    def _build_images(self):
        source = os.path.join(self._cwd, 'src', 'style', 'images')
        dest = os.path.join(self._config['build_path'], 'style', 'images')

        if not os.path.exists(source):
            return

        if not os.path.exists(self._project_build_style_path):
            try:
                os.makedirs(self._project_build_style_path)
            except:
                raise CreateFolderError('Could not create the style folder.')

        if os.path.exists(dest):
            try:
                rmtree(dest)
            except:
                raise RemoveFolderError('Could not remove the existing images folder.')

        try:
            copytree(source, dest)
        except:
            raise FileNotWritableError('Could not copy all the images in the image folder.')

    def _build_libraries(self):
        source = os.path.join(self._cwd, 'src', 'lib')
        dest = os.path.join(self._config['build_path'], 'lib')

        if not os.path.exists(source):
            return

        if os.path.exists(dest):
            try:
                rmtree(dest)
            except:
                raise RemoveFolderError('Could not remove the existing libraries folder.')

        try:
            copytree(source, dest)
        except:
            raise FileNotWritableError('Could not copy all the libraries.')


def clean():
    if not os.path.exists(os.path.join(os.getcwd(), 'build')):
        return

    try:
        rmtree(os.path.join(os.getcwd(), 'build'))
    except:
        raise RemoveFolderError('Could not remove the build folder.')



class Error(Exception):
    def __init__(self, msg='', arg=None):
        if arg:
            self.msg = msg + arg
        else:
            self.msg = msg

    def __repr__(self):
        return self.msg


class FileNotFoundError(Error):
    pass


class FileNotWritableError(Error):
    pass


class FolderAlreadyExistsError(Error):
    pass


class CreateFolderError(Error):
    pass


class RemoveFolderError(Error):
    pass


class FolderNotFoundError(Error):
    pass


class WrongFormatError(Error):
    pass


class MissingKeyError(Error):
    pass


class RemoveFileError(Error):
    pass


class UnknownCommandError(Error):
    pass


class SassError(Error):
    pass



from build import Build, clean
from deploy import Deploy
from zipit import Zip
from testit import Test
from create import New
import os
import json
import re
from error import FileNotFoundError, WrongFormatError, MissingKeyError, UnknownCommandError
import sys


def we_are_frozen():
    # All of the modules are built-in to the interpreter, e.g., by py2exe
    return hasattr(sys, "frozen")


def get_path():
    encoding = sys.getfilesystemencoding()
    if we_are_frozen():
        return os.path.dirname(unicode(sys.executable, encoding))
    return os.path.dirname(unicode(__file__, encoding))


class Task:
    def __init__(self, args):
        self._new = False
        self._build = False
        self._test = False
        self._deploy = False
        self._zip = False
        self._clean = False
        self._bad = False
        self._restrict = []
        self._args = args
        self._root = get_path()

        if args.new:
            self._new = True
            self._name = args.name
            self._type = args.type
        elif args.clean:
            self._clean = True
        else:
            if args.build:
                self._build = True
            if args.test:
                self._test = True
            if args.deploy:
                self._deploy = True
            if args.zip:
                self._zip = True
            if args.bad:
                self._bad = True
                self._build = True
                self._test = True
                self._deploy = True
                self._zip = True

            if not self._build and not self._test and not self._deploy and not self._zip and not self._bad:
                raise UnknownCommandError()

            try:
                self._parse_config()
            except:
                raise

            if args.html:
                self._restrict.append('html')

            if args.js:
                self._restrict.append('js')

            if args.css:
                self._restrict.append('css')

            if args.img:
                self._restrict.append('img')

            if args.lib:
                self._restrict.append('lib')

    def _parse_config(self):
        cwd = os.getcwd()

        try:
            config_file = open(os.path.join(cwd, 'project.cfg'))
        except:
            raise FileNotFoundError('Could not find a config file in this directory.')

        strings = []
        for line in config_file:
            if not re.search('^\s*//.*', line):
                strings.append(line)

        try:
            self._config = json.loads(''.join(strings))
        except:
            raise WrongFormatError('The provided config file could not be parsed.')

        if 'name' not in self._config:
            raise MissingKeyError('Name of the project needs to be in the config file.')

        if 'version' not in self._config:
            raise MissingKeyError('Please specify a version in your config file.')

        if 'type' not in self._config:
            self._type = 'default'
        else:
            self._type = self._config['type']

        if self._build or not self._test and self._deploy or not self._test and self._zip:
            self._config['build'] = True
            self._config['build_path'] = os.path.join(cwd, 'build', self._config['name'])
        else:
            self._config['build'] = False

        if self._test:
            try:
                self._config['testname'] = self._args.testname
            except:
                self._config['testname'] = None

            self._config['test'] = True
            self._config['test_build_path'] = os.path.join(cwd, 'build', self._config['name'] + '_test')
        else:
            self._config['test'] = False

    def execute(self):
        if self._clean:
            try:
                clean()
            except:
                raise
            return

        if self._type != 'default':
            module = __import__(self._type + '.plugin')
            try:
                plugin = getattr(module.plugin, self._type.title())()
            except:
                raise
        else:
            plugin = None

        if self._new:
            try:
                New(self._name, plugin, self._type)
                print 'Created the project with name ' + self._name + '.'
            except:
                raise
            return

        if plugin:
            try:
                plugin.pass_config(self._config)
            except:
                raise

        try:
            b = Build(self._config)
            t = Test(self._config)
            d = Deploy(self._config)
            z = Zip(self._config)
        except:
            raise

        if self._build:
            try:
                b.build_project(self._restrict)
                if plugin:
                    plugin.after_build()
                print 'Successfully built the project.'
            except:
                raise

        if self._test:
            try:
                t.build_test()
                if plugin:
                    plugin.after_test()
                print 'Successfully built the tests.'
            except:
                raise

        if self._deploy:
            if not self._build and not self._test:
                try:
                    b.build_project(self._restrict)
                    if plugin:
                        plugin.after_build()
                    print 'Successfully built the project.'
                except:
                    raise

            try:
                d.deploy_project()
                if plugin:
                    plugin.after_deploy()
                print 'Successfully deployed the project.'
            except:
                raise

        if self._zip:
            if not self._build and not self._test:
                b.build_project(self._restrict)
                if plugin:
                    plugin.after_build()
                print 'Successfully built the project.'

            try:
                z.zip_project()
                if plugin:
                    plugin.after_zip()
                print 'Successfully zipped the project.'
            except:
                raise
